@@ -40,36 +40,102 @@ extern crate clap;
 #[macro_use]
 extern crate log;
 
-use std::net::{IpAddr, Ipv4Addr, SocketAddr, ToSocketAddrs};
-use std::path::{Path, PathBuf};
-use std::sync::{Arc, RwLock};
+use std::{
+    net::{IpAddr, Ipv4Addr, SocketAddr, ToSocketAddrs},
+    path::{Path, PathBuf},
+    sync::Arc,
+};
 
 use clap::{Arg, ArgMatches};
-use tokio::net::TcpListener;
-use tokio::net::UdpSocket;
-use tokio::runtime::{self, Runtime};
+use tokio::{
+    net::{TcpListener, UdpSocket},
+    runtime,
+};
 
-#[cfg(feature = "dnssec")]
-use trust_dns_client::rr::rdata::key::KeyUsage;
 use trust_dns_client::rr::Name;
-use trust_dns_server::authority::{AuthorityObject, Catalog, ZoneType};
 #[cfg(feature = "dns-over-tls")]
 use trust_dns_server::config::dnssec::{self, TlsCertConfig};
-use trust_dns_server::config::{Config, ZoneConfig};
-use trust_dns_server::logger;
-use trust_dns_server::server::ServerFuture;
-use trust_dns_server::store::file::{FileAuthority, FileConfig};
 #[cfg(feature = "resolver")]
 use trust_dns_server::store::forwarder::ForwardAuthority;
 #[cfg(feature = "sqlite")]
 use trust_dns_server::store::sqlite::{SqliteAuthority, SqliteConfig};
-use trust_dns_server::store::StoreConfig;
+use trust_dns_server::{
+    authority::{AuthorityObject, Catalog, ZoneType},
+    config::{Config, ZoneConfig},
+    store::{
+        file::{FileAuthority, FileConfig},
+        StoreConfig,
+    },
+};
+use trust_dns_server::{logger, server::ServerFuture};
+
+#[cfg(feature = "dnssec")]
+use {trust_dns_client::rr::rdata::key::KeyUsage, trust_dns_server::authority::DnssecAuthority};
+
+#[cfg(feature = "dnssec")]
+async fn load_keys<A, L>(
+    authority: &mut A,
+    zone_name: Name,
+    zone_config: &ZoneConfig,
+) -> Result<(), String>
+where
+    A: DnssecAuthority<Lookup = L>,
+    L: Send + Sync + Sized + 'static,
+{
+    if zone_config.is_dnssec_enabled() {
+        for key_config in zone_config.get_keys() {
+            info!(
+                "adding key to zone: {:?}, is_zsk: {}, is_auth: {}",
+                key_config.key_path(),
+                key_config.is_zone_signing_key(),
+                key_config.is_zone_update_auth()
+            );
+            if key_config.is_zone_signing_key() {
+                let zone_signer = key_config.try_into_signer(zone_name.clone()).map_err(|e| {
+                    format!("failed to load key: {:?} msg: {}", key_config.key_path(), e)
+                })?;
+                authority
+                    .add_zone_signing_key(zone_signer)
+                    .await
+                    .expect("failed to add zone signing key to authority");
+            }
+            if key_config.is_zone_update_auth() {
+                let update_auth_signer =
+                    key_config.try_into_signer(zone_name.clone()).map_err(|e| {
+                        format!("failed to load key: {:?} msg: {}", key_config.key_path(), e)
+                    })?;
+                let public_key = update_auth_signer
+                    .key()
+                    .to_sig0key_with_usage(update_auth_signer.algorithm(), KeyUsage::Host)
+                    .expect("failed to get sig0 key");
+                authority
+                    .add_update_auth_key(zone_name.clone(), public_key)
+                    .await
+                    .expect("failed to add update auth key to authority");
+            }
+        }
+
+        info!("signing zone: {}", zone_config.get_zone().unwrap());
+        authority.secure_zone().await.expect("failed to sign zone");
+    }
+    Ok(())
+}
+
+#[cfg(not(feature = "dnssec"))]
+#[allow(clippy::unnecessary_wraps)]
+async fn load_keys<T>(
+    _authority: &mut T,
+    _zone_name: Name,
+    _zone_config: &ZoneConfig,
+) -> Result<(), String> {
+    Ok(())
+}
 
 #[cfg_attr(not(feature = "dnssec"), allow(unused_mut, unused))]
-fn load_zone(
+#[warn(clippy::wildcard_enum_match_arm)] // make sure all cases are handled despite of non_exhaustive
+async fn load_zone(
     zone_dir: &Path,
     zone_config: &ZoneConfig,
-    runtime: &mut Runtime,
 ) -> Result<Box<dyn AuthorityObject>, String> {
     debug!("loading zone with config: {:#?}", zone_config);
 
@@ -86,14 +152,14 @@ fn load_zone(
     }
 
     // load the zone
-    let mut authority: Box<dyn AuthorityObject> = match zone_config.stores {
+    let authority: Box<dyn AuthorityObject> = match zone_config.stores {
         #[cfg(feature = "sqlite")]
         Some(StoreConfig::Sqlite(ref config)) => {
             if zone_path.is_some() {
                 warn!("ignoring [[zones.file]] instead using [[zones.stores.zone_file_path]]");
             }
 
-            SqliteAuthority::try_from_config(
+            let mut authority = SqliteAuthority::try_from_config(
                 zone_name,
                 zone_type,
                 is_axfr_allowed,
@@ -101,27 +167,35 @@ fn load_zone(
                 Some(zone_dir),
                 config,
             )
-            .map(|a| Box::new(Arc::new(RwLock::new(a))))?
+            .await?;
+
+            // load any keys for the Zone, if it is a dynamic update zone, then keys are required
+            load_keys(&mut authority, zone_name_for_signer, zone_config).await?;
+            Box::new(Arc::new(authority)) as Box<dyn AuthorityObject>
         }
         Some(StoreConfig::File(ref config)) => {
             if zone_path.is_some() {
                 warn!("ignoring [[zones.file]] instead using [[zones.stores.zone_file_path]]");
             }
-            FileAuthority::try_from_config(
+
+            let mut authority = FileAuthority::try_from_config(
                 zone_name,
                 zone_type,
                 is_axfr_allowed,
                 Some(zone_dir),
                 config,
-            )
-            .map(|a| Box::new(Arc::new(RwLock::new(a))))?
+            )?;
+
+            // load any keys for the Zone, if it is a dynamic update zone, then keys are required
+            load_keys(&mut authority, zone_name_for_signer, zone_config).await?;
+            Box::new(Arc::new(authority)) as Box<dyn AuthorityObject>
         }
         #[cfg(feature = "resolver")]
         Some(StoreConfig::Forward(ref config)) => {
             let forwarder = ForwardAuthority::try_from_config(zone_name, zone_type, config);
-            let forwarder = runtime.block_on(forwarder)?;
+            let authority = forwarder.await?;
 
-            Box::new(Arc::new(RwLock::new(forwarder)))
+            Box::new(Arc::new(authority)) as Box<dyn AuthorityObject>
         }
         #[cfg(feature = "sqlite")]
         None if zone_config.is_update_allowed() => {
@@ -141,7 +215,7 @@ fn load_zone(
                 allow_update: zone_config.is_update_allowed(),
             };
 
-            SqliteAuthority::try_from_config(
+            let mut authority = SqliteAuthority::try_from_config(
                 zone_name,
                 zone_type,
                 is_axfr_allowed,
@@ -149,80 +223,34 @@ fn load_zone(
                 Some(zone_dir),
                 &config,
             )
-            .map(|a| Box::new(Arc::new(RwLock::new(a))))?
+            .await?;
+
+            // load any keys for the Zone, if it is a dynamic update zone, then keys are required
+            load_keys(&mut authority, zone_name_for_signer, zone_config).await?;
+            Box::new(Arc::new(authority)) as Box<dyn AuthorityObject>
         }
         None => {
             let config = FileConfig {
                 zone_file_path: zone_path.ok_or("file is a necessary parameter of zone_config")?,
             };
-            FileAuthority::try_from_config(
+
+            let mut authority = FileAuthority::try_from_config(
                 zone_name,
                 zone_type,
                 is_axfr_allowed,
                 Some(zone_dir),
                 &config,
-            )
-            .map(|a| Box::new(Arc::new(RwLock::new(a))))?
+            )?;
+
+            // load any keys for the Zone, if it is a dynamic update zone, then keys are required
+            load_keys(&mut authority, zone_name_for_signer, zone_config).await?;
+            Box::new(Arc::new(authority)) as Box<dyn AuthorityObject>
+        }
+        Some(_) => {
+            panic!("unrecognized authority type, check enabled features");
         }
     };
 
-    #[cfg(feature = "dnssec")]
-    fn load_keys(
-        authority: &mut dyn AuthorityObject,
-        zone_name: Name,
-        zone_config: &ZoneConfig,
-    ) -> Result<(), String> {
-        if zone_config.is_dnssec_enabled() {
-            for key_config in zone_config.get_keys() {
-                info!(
-                    "adding key to zone: {:?}, is_zsk: {}, is_auth: {}",
-                    key_config.key_path(),
-                    key_config.is_zone_signing_key(),
-                    key_config.is_zone_update_auth()
-                );
-                if key_config.is_zone_signing_key() {
-                    let zone_signer =
-                        key_config.try_into_signer(zone_name.clone()).map_err(|e| {
-                            format!("failed to load key: {:?} msg: {}", key_config.key_path(), e)
-                        })?;
-                    authority
-                        .add_zone_signing_key(zone_signer)
-                        .expect("failed to add zone signing key to authority");
-                }
-                if key_config.is_zone_update_auth() {
-                    let update_auth_signer =
-                        key_config.try_into_signer(zone_name.clone()).map_err(|e| {
-                            format!("failed to load key: {:?} msg: {}", key_config.key_path(), e)
-                        })?;
-                    let public_key = update_auth_signer
-                        .key()
-                        .to_sig0key_with_usage(update_auth_signer.algorithm(), KeyUsage::Host)
-                        .expect("failed to get sig0 key");
-                    authority
-                        .add_update_auth_key(zone_name.clone(), public_key)
-                        .expect("failed to add update auth key to authority");
-                }
-            }
-
-            info!("signing zone: {}", zone_config.get_zone().unwrap());
-            authority.secure_zone().expect("failed to sign zone");
-        }
-        Ok(())
-    }
-
-    #[cfg(not(feature = "dnssec"))]
-    #[allow(clippy::unnecessary_wraps)]
-    fn load_keys(
-        _authority: &mut dyn AuthorityObject,
-        _zone_name: Name,
-        _zone_config: &ZoneConfig,
-    ) -> Result<(), String> {
-        Ok(())
-    }
-
-    // load any keys for the Zone, if it is a dynamic update zone, then keys are required
-    load_keys(authority.as_mut(), zone_name_for_signer, zone_config)?;
-
     info!(
         "zone successfully loaded: {}",
         zone_config.get_zone().unwrap()
@@ -251,8 +279,8 @@ struct Args {
     pub(crate) flag_https_port: Option<u16>,
 }
 
-impl<'a> From<ArgMatches<'a>> for Args {
-    fn from(matches: ArgMatches<'a>) -> Args {
+impl<'a> From<ArgMatches> for Args {
+    fn from(matches: ArgMatches) -> Args {
         Args {
             flag_quiet: matches.is_present(QUIET_ARG),
             flag_debug: matches.is_present(DEBUG_ARG),
@@ -263,13 +291,13 @@ impl<'a> From<ArgMatches<'a>> for Args {
             flag_zonedir: matches.value_of(ZONEDIR_ARG).map(ToString::to_string),
             flag_port: matches
                 .value_of(PORT_ARG)
-                .map(|s| s.parse::<u16>().expect("bad port argument")),
+                .map(|s| s.parse().expect("bad port argument")),
             flag_tls_port: matches
                 .value_of(TLS_PORT_ARG)
-                .map(|s| s.parse::<u16>().expect("bad tls-port argument")),
+                .map(|s| s.parse().expect("bad tls-port argument")),
             flag_https_port: matches
                 .value_of(HTTPS_PORT_ARG)
-                .map(|s| s.parse::<u16>().expect("bad https-port argument")),
+                .map(|s| s.parse().expect("bad https-port argument")),
         }
     }
 }
@@ -277,52 +305,53 @@ impl<'a> From<ArgMatches<'a>> for Args {
 /// Main method for running the named server.
 ///
 /// `Note`: Tries to avoid panics, in favor of always starting.
+#[allow(unused_mut)]
 fn main() {
-    let args = app_from_crate!()
+    let args = command!()
         .arg(
-            Arg::with_name(QUIET_ARG)
+            Arg::new(QUIET_ARG)
                 .long(QUIET_ARG)
-                .short("q")
+                .short('q')
                 .help("Disable INFO messages, WARN and ERROR will remain")
                 .conflicts_with(DEBUG_ARG),
         )
         .arg(
-            Arg::with_name(DEBUG_ARG)
+            Arg::new(DEBUG_ARG)
                 .long(DEBUG_ARG)
-                .short("d")
+                .short('d')
                 .help("Turn on DEBUG messages (default is only INFO)")
                 .conflicts_with(QUIET_ARG),
         )
         .arg(
-            Arg::with_name(CONFIG_ARG)
+            Arg::new(CONFIG_ARG)
                 .long(CONFIG_ARG)
-                .short("c")
+                .short('c')
                 .help("Path to configuration file")
                 .value_name("FILE")
                 .default_value("/etc/named.toml"),
         )
         .arg(
-            Arg::with_name(ZONEDIR_ARG)
+            Arg::new(ZONEDIR_ARG)
                 .long(ZONEDIR_ARG)
-                .short("z")
+                .short('z')
                 .help("Path to the root directory for all zone files, see also config toml")
                 .value_name("DIR"),
         )
         .arg(
-            Arg::with_name(PORT_ARG)
+            Arg::new(PORT_ARG)
                 .long(PORT_ARG)
-                .short("p")
+                .short('p')
                 .help("Listening port for DNS queries, overrides any value in config file")
                 .value_name(PORT_ARG),
         )
         .arg(
-            Arg::with_name(TLS_PORT_ARG)
+            Arg::new(TLS_PORT_ARG)
                 .long(TLS_PORT_ARG)
                 .help("Listening port for DNS over TLS queries, overrides any value in config file")
                 .value_name(TLS_PORT_ARG),
         )
         .arg(
-            Arg::with_name(HTTPS_PORT_ARG)
+            Arg::new(HTTPS_PORT_ARG)
                 .long(HTTPS_PORT_ARG)
                 .help(
                     "Listening port for DNS over HTTPS queries, overrides any value in config file",
@@ -371,7 +400,7 @@ fn main() {
             .get_zone()
             .unwrap_or_else(|_| panic!("bad zone name in {:?}", config_path));
 
-        match load_zone(&zone_dir, zone, &mut runtime) {
+        match runtime.block_on(load_zone(&zone_dir, zone)) {
             Ok(authority) => catalog.upsert(zone_name.into(), authority),
             Err(error) => panic!("could not load zone {}: {}", zone_name, error),
         }
@@ -501,7 +530,7 @@ fn config_tls(
     tls_cert_config: &TlsCertConfig,
     zone_dir: &Path,
     listen_addrs: &[IpAddr],
-    runtime: &mut Runtime,
+    runtime: &mut runtime::Runtime,
 ) {
     use futures::TryFutureExt;
 
@@ -554,7 +583,7 @@ fn config_https(
     tls_cert_config: &TlsCertConfig,
     zone_dir: &Path,
     listen_addrs: &[IpAddr],
-    runtime: &mut Runtime,
+    runtime: &mut runtime::Runtime,
 ) {
     use futures::TryFutureExt;
 
